set debugon 0

proc MicOff {} {
    snd_setlevel "mic" 0
}

proc MicOn { {level 80} } {
    snd_setlevel "mic" $level
}

proc SpkrOn { {level 80}} {
    snd_setlevel "vol" $level
}

proc SpkrOff {} {
    snd_setlevel "vol" 0
}

proc GetQuietLevel { maxlen level } {
    SpkrOff
    MicOn $level
    return [snd_getquietlevel $maxlen]
}

proc reco { } {
    SpkrOff
    MicOn 
    set buffer [snd_voxalloc 5 16384]
    set buflen [snd_record $buffer 5 16384]
    snd_savefile testing $buffer [expr 5 * 16384 * 2]
    snd_freefile $buffer
}

proc Record { maxlen rate level} {
    put_debug "Record $maxlen $rate $level"

    SpkrOff
    MicOn $level
    set buffer [snd_voxalloc $maxlen $rate]
    set buflen [snd_voxrecord $buffer $maxlen  $rate]
    MicOff

    return "$buffer $buflen"
}

proc RecordToFile { filename maxlen {rate 8192} {level 80} } {
    put_debug "RecordToFile $filename $maxlen $rate $level"

    SpkrOff
    MicOn $level
    snd_voxrecordfile $filename $maxlen  $rate
}

proc Play { buffer len rate level} {
    put_debug "Play $buffer $len $rate $level"

    MicOff
    SpkrOn $level
    snd_play $buffer $len $rate
}

proc PlayStart { buffer len rate level} {
    global playfd

    put_debug "PlayStart $buffer $len $rate $level"

    MicOff
    SpkrOn $level
    set playfd [snd_playstart $buffer $len $rate]
}

proc PlayStop { } {
    global playfd

    put_debug "PlayStop"

    snd_playstop $playfd
}

proc SetVoxLevel { level } {
    global snd_voxlevel
    put_debug "SetVoxLevel $level"

    set snd_voxlevel $level
}

proc LoadSoundFile { filename reflen } {
    upvar $relen len
    put_debug "LoadSoundFile $filename"
    
    set result [snd_loadfile $filename] 
    scan $result "%d%d" addr len
    return $addr
}

proc PlayFile { filename {rate 8192} {level 80}} {
    put_debug "PlayFile $filename $rate $level"

    set result [snd_loadfile $filename]
    scan $result "%d%d" addr len
    Play $addr $len $rate $level
    snd_freefile $addr
}

proc PlayFileStart { filename {rate 8192} {level 80}} {
    global playaddr

    put_debug "PlayFileStart $filename $rate $level"
    
    set result [snd_loadfile $filename]
    scan $result "%d%d" playaddr len
    PlayStart $playaddr $len $rate $level
}

proc PlayFileStop { } {
    global playaddr

    put_debug "PlayFileStop"

    PlayStop
    snd_freefile $playaddr
}

proc Calibrate { {rec_rate 8192} {rec_level 80} } {
    # Initialization phase (measure silence)
    MicOn $rec_level
    SpkrOff
    set level 0
    while { $level > 160 || $level < 135 } {
	set level [snd_getquietlevel 2 $rec_rate]
	puts -nonewline $level
	flush stdout
    }
    set level [expr $level + 10]
    SetVoxLevel $level
    return $level
}

proc put_debug { msg } {
    global debugon
    if $debugon {
	puts $msg
    }
}